@@ -11,7 +11,6 @@ use crate::{
             strip_trivia, FormatTriviaType, UpdateLeadingTrivia, UpdateTrailingTrivia, UpdateTrivia,
         },
         trivia_util,
-        util::token_range,
     },
     shape::Shape,
 };
@@ -32,39 +31,48 @@ macro_rules! update_first_token {
 }
 
 pub fn format_return<'ast>(
-    ctx: &mut Context,
+    ctx: &Context,
     return_node: &Return<'ast>,
     shape: Shape,
 ) -> Return<'ast> {
     // Calculate trivia
-    let additional_indent_level = ctx.get_range_indent_increase(token_range(return_node.token()));
-    let shape = shape.with_additional_indent(additional_indent_level);
-    let leading_trivia = vec![create_indent_trivia(ctx, additional_indent_level)];
+    let leading_trivia = vec![create_indent_trivia(ctx, shape)];
     let trailing_trivia = vec![create_newline_trivia(ctx)];
 
     if return_node.returns().is_empty() {
-        let token = fmt_symbol!(ctx, return_node.token(), "return").update_trivia(
+        let token = fmt_symbol!(ctx, return_node.token(), "return", shape).update_trivia(
             FormatTriviaType::Append(leading_trivia),
             FormatTriviaType::Append(trailing_trivia),
         );
 
         Return::new().with_token(token)
     } else {
-        let token = fmt_symbol!(ctx, return_node.token(), "return ")
+        let token = fmt_symbol!(ctx, return_node.token(), "return ", shape)
             .update_leading_trivia(FormatTriviaType::Append(leading_trivia));
 
         let shape = shape + (strip_trivia(return_node.token()).to_string().len() + 1); // 1 = " "
-        let mut formatted_returns =
-            try_format_punctuated(ctx, return_node.returns(), shape, format_expression);
+        let mut formatted_returns = try_format_punctuated(
+            ctx,
+            return_node.returns(),
+            shape,
+            format_expression,
+            Some(1),
+        );
 
         // Determine if we need to hang the condition
         let require_multiline_expression = shape
             .take_first_line(&strip_trivia(&formatted_returns))
-            .over_budget();
+            .over_budget()
+            || {
+                trivia_util::contains_comments(
+                    return_node
+                        .returns()
+                        .update_trailing_trivia(FormatTriviaType::Replace(Vec::new())), // We can ignore trailing trivia, as that won't affect anything
+                )
+            };
 
         if require_multiline_expression {
-            formatted_returns =
-                hang_punctuated_list(ctx, return_node.returns(), shape, additional_indent_level);
+            formatted_returns = hang_punctuated_list(ctx, return_node.returns(), shape);
         }
 
         if let Some(pair) = formatted_returns.pop() {
@@ -80,20 +88,19 @@ pub fn format_return<'ast>(
 }
 
 pub fn format_last_stmt<'ast>(
-    ctx: &mut Context,
+    ctx: &Context,
     last_stmt: &LastStmt<'ast>,
     shape: Shape,
 ) -> LastStmt<'ast> {
     check_should_format!(ctx, last_stmt);
 
     match last_stmt {
-        LastStmt::Break(token) => LastStmt::Break(fmt_symbol!(ctx, token, "break").update_trivia(
-            FormatTriviaType::Append(vec![create_indent_trivia(
-                ctx,
-                ctx.get_range_indent_increase(token_range(token)),
-            )]),
-            FormatTriviaType::Append(vec![create_newline_trivia(ctx)]),
-        )),
+        LastStmt::Break(token) => {
+            LastStmt::Break(fmt_symbol!(ctx, token, "break", shape).update_trivia(
+                FormatTriviaType::Append(vec![create_indent_trivia(ctx, shape)]),
+                FormatTriviaType::Append(vec![create_newline_trivia(ctx)]),
+            ))
+        }
 
         LastStmt::Return(return_node) => LastStmt::Return(format_return(ctx, return_node, shape)),
         #[cfg(feature = "luau")]
@@ -108,12 +115,10 @@ pub fn format_last_stmt<'ast>(
                     }),
                     vec![],
                 ),
+                shape,
             )
             .update_trivia(
-                FormatTriviaType::Append(vec![create_indent_trivia(
-                    ctx,
-                    ctx.get_range_indent_increase(token_range(token)),
-                )]),
+                FormatTriviaType::Append(vec![create_indent_trivia(ctx, shape)]),
                 FormatTriviaType::Append(vec![create_newline_trivia(ctx)]),
             ),
         ),
@@ -309,12 +314,14 @@ fn last_stmt_remove_leading_newlines(last_stmt: LastStmt) -> LastStmt {
     }
 }
 
-pub fn format_block<'ast>(ctx: &mut Context, block: Block<'ast>) -> Block<'ast> {
+/// Formats a block node. Note: the given shape to the block formatter should already be at the correct indentation level
+pub fn format_block<'ast>(ctx: &Context, block: &Block<'ast>, shape: Shape) -> Block<'ast> {
     let mut formatted_statements: Vec<(Stmt<'ast>, Option<TokenReference<'ast>>)> = Vec::new();
     let mut found_first_stmt = false;
     let mut stmt_iterator = block.stmts_with_semicolon().peekable();
+
     while let Some((stmt, semi)) = stmt_iterator.next() {
-        let shape = Shape::from_context(ctx);
+        let shape = shape.reset();
         let mut stmt = format_stmt(ctx, stmt, shape);
 
         // If this is the first stmt, then remove any leading newlines
@@ -339,6 +346,17 @@ pub fn format_block<'ast>(ctx: &mut Context, block: Block<'ast>) -> Block<'ast>
                         function_call.prefix(),
                         Prefix::Expression(Expression::Parentheses { .. })
                     ),
+                    Some((Stmt::Assignment(assignment), _)) => {
+                        match assignment.variables().iter().next() {
+                            Some(Var::Expression(var_expression)) => {
+                                matches!(
+                                    var_expression.prefix(),
+                                    Prefix::Expression(Expression::Parentheses { .. })
+                                )
+                            }
+                            _ => false,
+                        }
+                    }
                     _ => false,
                 }
             }
@@ -353,7 +371,7 @@ pub fn format_block<'ast>(ctx: &mut Context, block: Block<'ast>) -> Block<'ast>
                 stmt = updated_stmt;
                 Some(
                     match semi {
-                        Some(semi) => crate::fmt_symbol!(ctx, semi, ";"),
+                        Some(semi) => crate::fmt_symbol!(ctx, semi, ";", shape),
                         None => TokenReference::symbol(";").expect("could not make semicolon"),
                     }
                     .update_trailing_trivia(FormatTriviaType::Append(trivia)),
@@ -371,6 +389,7 @@ pub fn format_block<'ast>(ctx: &mut Context, block: Block<'ast>) -> Block<'ast>
                             ctx,
                             semi,
                             &TokenReference::new(vec![], Token::new(TokenType::spaces(0)), vec![]),
+                            shape,
                         )
                         .update_trailing_trivia(FormatTriviaType::Append(trivia)),
                     )
@@ -387,7 +406,7 @@ pub fn format_block<'ast>(ctx: &mut Context, block: Block<'ast>) -> Block<'ast>
 
     let formatted_last_stmt = match block.last_stmt_with_semicolon() {
         Some((last_stmt, semi)) => {
-            let shape = Shape::from_context(ctx);
+            let shape = shape.reset();
             let mut last_stmt = format_last_stmt(ctx, last_stmt, shape);
             // If this is the first stmt, then remove any leading newlines
             if !found_first_stmt && ctx.should_format_node(&last_stmt) {
@@ -407,6 +426,7 @@ pub fn format_block<'ast>(ctx: &mut Context, block: Block<'ast>) -> Block<'ast>
                         ctx,
                         semi,
                         &TokenReference::new(vec![], Token::new(TokenType::spaces(0)), vec![]),
+                        shape,
                     )
                     .update_trailing_trivia(FormatTriviaType::Append(trivia));
                     Some(semicolon_token)
@@ -419,6 +439,7 @@ pub fn format_block<'ast>(ctx: &mut Context, block: Block<'ast>) -> Block<'ast>
     };
 
     block
+        .to_owned()
         .with_stmts(formatted_statements)
         .with_last_stmt(formatted_last_stmt)
 }
