@@ -2,7 +2,8 @@ use crate::formatters::trivia::{FormatTriviaType, UpdateTrailingTrivia};
 #[cfg(feature = "luau")]
 use full_moon::ast::span::ContainedSpan;
 #[cfg(feature = "luau")]
-use full_moon::ast::types::{IndexedTypeInfo, TypeDeclaration, TypeInfo};
+use full_moon::ast::types::{IndexedTypeInfo, TypeDeclaration, TypeField, TypeInfo};
+use full_moon::ast::Block;
 use full_moon::{
     ast::{
         BinOp, Call, Expression, Field, FunctionArgs, Index, LastStmt, Prefix, Stmt, Suffix,
@@ -13,7 +14,14 @@ use full_moon::{
 };
 
 pub fn trivia_is_whitespace(trivia: &Token) -> bool {
-    matches!(trivia.token_type(), TokenType::Whitespace { .. })
+    matches!(trivia.token_kind(), TokenKind::Whitespace)
+}
+
+pub fn trivia_is_comment(trivia: &Token) -> bool {
+    matches!(
+        trivia.token_kind(),
+        TokenKind::SingleLineComment | TokenKind::MultiLineComment
+    )
 }
 
 pub fn trivia_is_newline(trivia: &Token) -> bool {
@@ -39,11 +47,26 @@ pub fn can_hang_expression(expression: &Expression) -> bool {
         Expression::Parentheses { expression, .. } => can_hang_expression(expression),
         Expression::UnaryOperator { expression, .. } => can_hang_expression(expression),
         Expression::BinaryOperator { .. } => true, // If a binop is present, then we can hang the expression
-        Expression::Value { .. } => false,
+        Expression::Value { value, .. } => match &**value {
+            Value::ParenthesesExpression(expression) => can_hang_expression(expression),
+            Value::FunctionCall(function_call) => match function_call.prefix() {
+                Prefix::Expression(expression) => can_hang_expression(expression),
+                _ => false,
+            },
+            Value::Var(Var::Expression(expression)) => match expression.prefix() {
+                Prefix::Expression(expression) => can_hang_expression(expression),
+                _ => false,
+            },
+            _ => false,
+        },
         other => panic!("unknown node {:?}", other),
     }
 }
 
+pub fn is_block_empty(block: &Block) -> bool {
+    block.stmts().next().is_none() && block.last_stmt().is_none()
+}
+
 // TODO: Can we clean this up? A lot of this code is repeated in trivia_formatter
 fn function_args_trailing_trivia<'ast>(function_args: &FunctionArgs<'ast>) -> Vec<Token<'ast>> {
     match function_args {
@@ -141,6 +164,7 @@ fn type_info_trailing_trivia<'ast>(type_info: &TypeInfo<'ast>) -> Vec<Token<'ast
         }
 
         TypeInfo::Union { right, .. } => type_info_trailing_trivia(right),
+        TypeInfo::Variadic { type_info, .. } => type_info_trailing_trivia(type_info),
 
         other => panic!("unknown node {:?}", other),
     }
@@ -282,6 +306,31 @@ pub fn get_expression_leading_trivia<'ast>(expression: &Expression<'ast>) -> Vec
     }
 }
 
+pub fn binop_leading_comments<'ast>(binop: &BinOp<'ast>) -> Vec<Token<'ast>> {
+    match binop {
+        BinOp::And(token)
+        | BinOp::Caret(token)
+        | BinOp::GreaterThan(token)
+        | BinOp::GreaterThanEqual(token)
+        | BinOp::LessThan(token)
+        | BinOp::LessThanEqual(token)
+        | BinOp::Minus(token)
+        | BinOp::Or(token)
+        | BinOp::Percent(token)
+        | BinOp::Plus(token)
+        | BinOp::Slash(token)
+        | BinOp::Star(token)
+        | BinOp::TildeEqual(token)
+        | BinOp::TwoDots(token)
+        | BinOp::TwoEqual(token) => token
+            .leading_trivia()
+            .filter(|token| trivia_is_comment(token))
+            .map(|x| x.to_owned())
+            .collect(),
+        other => panic!("unknown node {:?}", other),
+    }
+}
+
 pub fn binop_trailing_comments<'ast>(binop: &BinOp<'ast>) -> Vec<Token<'ast>> {
     match binop {
         BinOp::And(token)
@@ -301,30 +350,31 @@ pub fn binop_trailing_comments<'ast>(binop: &BinOp<'ast>) -> Vec<Token<'ast>> {
         | BinOp::TwoEqual(token) => {
             token
                 .trailing_trivia()
-                .filter(|token| {
-                    token.token_kind() == TokenKind::SingleLineComment
-                        || token.token_kind() == TokenKind::MultiLineComment
-                })
-                .map(|x| {
+                .filter(|token| trivia_is_comment(token))
+                .flat_map(|x| {
                     // Prepend a single space beforehand
                     vec![Token::new(TokenType::spaces(1)), x.to_owned()]
                 })
-                .flatten()
                 .collect()
         }
         other => panic!("unknown node {:?}", other),
     }
 }
 
-pub fn get_expression_trailing_comments<'ast>(
+pub fn expression_leading_comments<'ast>(expression: &Expression<'ast>) -> Vec<Token<'ast>> {
+    get_expression_leading_trivia(expression)
+        .iter()
+        .filter(|token| trivia_is_comment(token))
+        .map(|x| x.to_owned())
+        .collect()
+}
+
+pub fn take_expression_trailing_comments<'ast>(
     expression: &Expression<'ast>,
 ) -> (Expression<'ast>, Vec<Token<'ast>>) {
     let trailing_comments = get_expression_trailing_trivia(expression)
         .iter()
-        .filter(|token| {
-            token.token_kind() == TokenKind::SingleLineComment
-                || token.token_kind() == TokenKind::MultiLineComment
-        })
+        .filter(|token| trivia_is_comment(token))
         .map(|x| {
             // Prepend a single space beforehand
             vec![Token::new(TokenType::spaces(1)), x.to_owned()]
@@ -340,6 +390,28 @@ pub fn get_expression_trailing_comments<'ast>(
     )
 }
 
+#[cfg(feature = "luau")]
+pub fn take_type_field_trailing_comments(
+    type_field: TypeField<'_>,
+) -> (TypeField<'_>, Vec<Token<'_>>) {
+    let trailing_comments = type_info_trailing_trivia(type_field.value())
+        .iter()
+        .filter(|token| trivia_is_comment(token))
+        .map(|x| {
+            // Prepend a single space beforehand
+            vec![Token::new(TokenType::spaces(1)), x.to_owned()]
+        })
+        .flatten()
+        .collect();
+
+    (
+        type_field.update_trailing_trivia(
+            FormatTriviaType::Replace(vec![]), // TODO: Do we need to keep some trivia?
+        ),
+        trailing_comments,
+    )
+}
+
 /// Macro for retrieving trailing trivia out of a stmt which ends with an `end` token
 macro_rules! end_stmt_trailing_trivia {
     ($enum:ident, $value:ident) => {{
@@ -551,6 +623,16 @@ fn get_type_info_trailing_trivia(type_info: TypeInfo) -> (TypeInfo, Vec<Token>)
                 trailing_trivia,
             )
         }
+        TypeInfo::Variadic { ellipse, type_info } => {
+            let (type_info, trailing_trivia) = get_type_info_trailing_trivia(*type_info);
+            (
+                TypeInfo::Variadic {
+                    ellipse,
+                    type_info: Box::new(type_info),
+                },
+                trailing_trivia,
+            )
+        }
         other => panic!("unknown node {:?}", other),
     }
 }
@@ -672,10 +754,7 @@ pub fn get_stmt_trailing_trivia(stmt: Stmt) -> (Stmt, Vec<Token>) {
             let new_end_token = end_token.update_trailing_trivia(FormatTriviaType::Replace(vec![]));
 
             let body = stmt.body().to_owned().with_end_token(new_end_token);
-            (
-                Stmt::LocalFunction(stmt.with_func_body(body)),
-                trailing_trivia,
-            )
+            (Stmt::LocalFunction(stmt.with_body(body)), trailing_trivia)
         }
         Stmt::NumericFor(stmt) => {
             end_stmt_trailing_trivia!(NumericFor, stmt)
@@ -798,15 +877,21 @@ pub fn token_trivia_contains_comments<'ast>(
     trivia: impl Iterator<Item = &'ast Token<'ast>>,
 ) -> bool {
     for trivia in trivia {
-        if trivia.token_kind() == TokenKind::SingleLineComment
-            || trivia.token_kind() == TokenKind::MultiLineComment
-        {
+        if trivia_is_comment(trivia) {
             return true;
         }
     }
     false
 }
 
+pub fn token_contains_leading_comments(token_ref: &TokenReference) -> bool {
+    token_trivia_contains_comments(token_ref.leading_trivia())
+}
+
+pub fn token_contains_trailing_comments(token_ref: &TokenReference) -> bool {
+    token_trivia_contains_comments(token_ref.trailing_trivia())
+}
+
 pub fn token_contains_comments(token_ref: &TokenReference) -> bool {
     token_trivia_contains_comments(token_ref.leading_trivia())
         || token_trivia_contains_comments(token_ref.trailing_trivia())
